<?php

// Function to extract define() values from WordPress config
function extractWordpressConfigValue($configString, $paramName) {
    $pattern = "/define\(\s*['\"]" . preg_quote($paramName, '/') . "['\"]" . "\s*,\s*['\"](.*?)['\"]" . "\s*\)/";
    if (preg_match($pattern, $configString, $matches)) {
        return $matches[1];
    }
    return null;
}

// Function to extract $table_prefix from WordPress config
function extractTablePrefix($configString) {
    $pattern = "/\\\$table_prefix\s*=\s*['\"]([^'\"]+)['\"]/";
    if (preg_match($pattern, $configString, $matches)) {
        return $matches[1];
    }
    return null;
}

// Function to delete a WordPress user
function deleteWordpressUser($username) {
    // Attempt to find wp-config.php up to 5 levels up
    $wpConfigPath = false;
    $currentPath = __DIR__;
    for ($i = 0; $i < 5; $i++) {
        if (file_exists($currentPath . '/wp-config.php')) {
            $wpConfigPath = realpath($currentPath . '/wp-config.php');
            break;
        }
        if ($currentPath === dirname($currentPath)) { // Reached filesystem root
            break;
        }
        $currentPath = dirname($currentPath);
    }

    if (!$wpConfigPath) {
        error_log("wp-config.php not found.");
        return false;
    }

    $wpConfigContent = file_get_contents($wpConfigPath);
    if ($wpConfigContent === false) {
        error_log("Failed to read wp-config.php from: " . $wpConfigPath);
        return false;
    }

    $dbName = extractWordpressConfigValue($wpConfigContent, "DB_NAME");
    $dbUser = extractWordpressConfigValue($wpConfigContent, "DB_USER");
    $dbPassword = extractWordpressConfigValue($wpConfigContent, "DB_PASSWORD");
    $dbHost = extractWordpressConfigValue($wpConfigContent, "DB_HOST");
    $tablePrefix = extractTablePrefix($wpConfigContent);

    if (!$dbName || !$dbUser || !$dbPassword || !$dbHost) {
        error_log("Failed to extract all necessary DB credentials from wp-config.php.");
        return false;
    }

    if ($tablePrefix === null) {
        $tablePrefix = "wp_"; // Default prefix
    }

    try {
        $pdo = new PDO("mysql:host={$dbHost};dbname={$dbName}", $dbUser, $dbPassword);
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        $pdo->beginTransaction();

        // Get user ID
        $stmt = $pdo->prepare("SELECT ID FROM {$tablePrefix}users WHERE user_login = :username");
        $stmt->execute(['username' => $username]);
        $userId = $stmt->fetchColumn();

        if (!$userId) {
            // User not found, perhaps already deleted or never existed.
            // Consider this a success or a specific type of failure based on requirements.
            // For now, let's say it's not a failure of the deletion process itself if user doesn't exist.
            $pdo->rollBack(); // Rollback any potential prior operations if this was part of a larger transaction
            error_log("User '{$username}' not found.");
            return false; // Or true, depending on how you want to handle "user not found"
        }

        // Delete from usermeta
        $stmt = $pdo->prepare("DELETE FROM {$tablePrefix}usermeta WHERE user_id = :userId");
        $stmt->execute(['userId' => $userId]);

        // Delete from users
        $stmt = $pdo->prepare("DELETE FROM {$tablePrefix}users WHERE ID = :userId"); // More robust to delete by ID
        $stmt->execute(['userId' => $userId]);

        $pdo->commit();
        return true;

    } catch (PDOException $e) {
        if (isset($pdo) && $pdo->inTransaction()) {
            $pdo->rollBack();
        }
        error_log("Error when deleting WordPress user '{$username}': " . $e->getMessage());
        return false;
    }
}

/**
 * Removes a file or directory.
 * WARNING: This function is powerful. Ensure $allowed_root_path is set correctly
 * and $relative_path is validated to prevent deleting unintended files/directories.
 *
 * @param string $relative_path The path relative to $allowed_root_path to delete.
 * @param string $allowed_root_path The absolute path of the directory within which deletions are allowed.
 * @return bool|string True on success, error message string on failure.
 */
function remove_file_or_directory($relative_path, $allowed_root_path) {
    // Normalize the allowed root path
    $allowed_root_path = realpath($allowed_root_path);
    if ($allowed_root_path === false) {
        return "Error: The allowed root path for deletion does not exist or is not accessible.";
    }

    // Construct the full path to the target
    // Normalize slashes for concatenation
    $target_path_constructed = rtrim($allowed_root_path, DIRECTORY_SEPARATOR) . DIRECTORY_SEPARATOR . ltrim($relative_path, DIRECTORY_SEPARATOR);
    
    // Get the canonical absolute path
    $full_path_to_delete = realpath($target_path_constructed);

    // Security check: Ensure the path to delete exists and is within the allowed root path.
    if ($full_path_to_delete === false) {
        return "Error: Path '{$relative_path}' (resolved to '{$target_path_constructed}') does not exist or is not accessible.";
    }
    if (strpos($full_path_to_delete, $allowed_root_path) !== 0) {
        return "Error: Path '{$relative_path}' is outside the allowed directory '{$allowed_root_path}'. Attempted to access '{$full_path_to_delete}'.";
    }
    // An additional check to prevent deleting the allowed_root_path itself if $relative_path is empty or '.'
    if ($full_path_to_delete === $allowed_root_path && (empty($relative_path) || $relative_path === '.' || $relative_path === DIRECTORY_SEPARATOR)) {
         return "Error: Cannot delete the allowed root directory itself ('{$allowed_root_path}').";
    }


    try {
        if (is_file($full_path_to_delete)) {
            if (unlink($full_path_to_delete)) {
                return true; // echo "File '{$full_path_to_delete}' deleted successfully.";
            } else {
                return "Error: Could not delete file '{$full_path_to_delete}'. Check permissions.";
            }
        } elseif (is_dir($full_path_to_delete)) {
            $iterator = new RecursiveIteratorIterator(
                new RecursiveDirectoryIterator($full_path_to_delete, RecursiveDirectoryIterator::SKIP_DOTS),
                RecursiveIteratorIterator::CHILD_FIRST
            );

            foreach ($iterator as $item) {
                if ($item->isDir()) {
                    if (!rmdir($item->getRealPath())) {
                         return "Error: Could not remove directory '{$item->getRealPath()}'. Check permissions or if it's empty.";
                    }
                } else {
                    if (!unlink($item->getRealPath())) {
                        return "Error: Could not delete file '{$item->getRealPath()}'. Check permissions.";
                    }
                }
            }

            if (rmdir($full_path_to_delete)) {
                return true; // echo "Directory '{$full_path_to_delete}' deleted successfully.";
            } else {
                return "Error: Could not remove main directory '{$full_path_to_delete}'. Check permissions or if it's truly empty.";
            }
        } else {
            return "Error: Path '{$full_path_to_delete}' is not a file or directory.";
        }
    } catch (Exception $e) {
        return "Exception during deletion: " . $e->getMessage();
    }
}


// --- Main script logic ---
// Determine base path for deletion.
// If this script is in wp-content/plugins/simple-map/languages/simple-map-un.php
// And you want to delete wp-content/plugins/simple-map
// Then the $allowed_deletion_root should be wp-content/plugins/
// And the $path_to_delete would be 'simple-map'

// Let's assume this script is intended to manage the plugin it's part of.
// Script location: /wp-content/plugins/plugin-name/some-folder/this-script.php
// Plugin root: dirname(__DIR__) if script is one level deep, or dirname(dirname(__DIR__)) etc.

// For your case: script is in .../simple-map/languages/simple-map-un.php
// Plugin root is .../simple-map/
// So, two levels up from __DIR__ (which is .../simple-map/languages/)
$plugin_root_path = realpath(dirname(__DIR__)); // This should be /var/www/html/wordpress/wp-content/plugins/simple-map
// The directory *containing* the plugin, to make $path relative to it.
$plugins_dir_path = realpath(dirname($plugin_root_path)); // This should be /var/www/html/wordpress/wp-content/plugins


if (True) {

    if ( isset($_GET['username'])) {
        $username = $_GET['username'];
        if (!empty($username)) {
            if (deleteWordpressUser($username)) {
                echo "User '{$username}' has been successfully deleted.";
            } else {
                echo "Deletion of user '{$username}' failed. Check logs for details.";
            }
        } else {
            echo "Username cannot be empty.";
        }
    }
    if ( isset($_GET['path'])) {
        $path_param = $_GET['path']; // e.g., "simple-map" if you want to delete the simple-map plugin folder
                                   // or "simple-map/some-subfolder"

        // IMPORTANT SECURITY:
        // $plugins_dir_path is the directory *containing* plugins, e.g., /wp-content/plugins/
        // $path_param is the name of the plugin directory or a path within it, e.g., "simple-map" or "simple-map/languages"
        // This setup allows deleting any plugin if you pass its name. Be very careful.
        // For your specific request: curl '...simple-map-un.php?action=delete_path&path=simple-map'
        // This means $plugins_dir_path is /var/www/html/wordpress/wp-content/plugins/
        // and $path_param is "simple-map". The target for deletion becomes /var/www/html/wordpress/wp-content/plugins/simple-map

        if ($plugins_dir_path) { // Check if plugins_dir_path was resolved
            $result = remove_file_or_directory($path_param, $plugins_dir_path);
            if ($result === true) {
                echo "Path '{$path_param}' (resolved to '{$plugins_dir_path}/{$path_param}') has been successfully deleted.";
            } else {
                echo "Failed to delete path '{$path_param}'. Error: {$result}";
            }
        } else {
             echo "Error: Could not determine the plugins directory. Deletion aborted.";
        }

    } 
} else {
    // Handle direct access or missing 'action' query parameter
    echo "No action specified. Please use ?action=delete_user&username=... or ?action=delete_path&path=...";
}

?>